<?php

namespace Drupal\cforge_group;

Use Drupal\user\Entity\User;
use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Batch\BatchBuilder;
use Drupal\Core\Mail\MailManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Send an email to all members of the site, regardless of preferences.
 */
class GroupMailForm extends EntityForm {

  protected $mailPluginManager;
  protected array $memberships;

  /**
   * @param MailManager $mail_plugin_manger
   */
  function __construct(MailManager $mail_plugin_manger) {
    $this->mailPluginManager = $mail_plugin_manger;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('plugin.manager.mail')
    );
  }

  public function init(FormStateInterface $form_state) {
    parent::init($form_state);
    $this->memberships = \Drupal::entityQuery('group_relationship')->accessCheck(TRUE)
      ->condition('type', 'project-group_membership')
      ->condition('gid', $this->entity->id())
      ->execute();
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'cforge_contact_all';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    parent::buildForm($form, $form_state);
    $form['subject'] = [
      '#title' => $this->t('Message subject'),
      '#type' => 'textfield',
      '#weight' => 1,
    ];
    $form['body'] = [
      '#title' => $this->t('Message body'),
      '#type' => 'text_format',
      '#format' => 'mail_authoring',
      '#rows' => 10,
      '#weight' => 2,
    ];

    $form['tokens'] = [
      '#theme' => 'token_tree_link',
      '#token_types' => ['group', 'site', 'current-user'],
      '#global_types' => FALSE,
      '#weight' => 3
    ];

    $form['test'] = [
      '#type' => 'submit',
      '#name' => 'test',
      '#value' => $this->t('Send yourself a test'),
      '#weight' => 4,
      '#prefix' => '<br />'
    ];
    $form['submit'] = [
      '#type' => 'submit',
      '#name' => 'send',
      '#value' => $this->t('Send to @count members', ['@count' => count($this->memberships)]),
      '#weight' => 5,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $clicked_button = $form_state->getTriggeringElement();
    if ($clicked_button['#name'] == 'test') {
      $this->batchSend([\Drupal::currentUser()->id()], $form_state->getValue('subject'), $form_state->getValue('body'));
    }
    else {
      $this->createBatch($form_state->getValues());
    }
  }


  function createBatch(array $form_values) {
    $memships = \Drupal\group\Entity\GroupRelationship::loadMultiple($this->memberships);
    foreach ($memships as $mem) {
      /** @var \Drupal\group\Entity\GroupRelationship $mem */
      $uids[] = $mem->getEntity()->id();
    }
    $batch_builder = (new BatchBuilder())
      ->setTitle($this->t('Sending @count mails', ['@count' => count($uids)]))
      ->setFile('profiles/cforge/modules/cforge_group/src/GroupMailForm.php')
      ->setProgressMessage($this->t('Processed @current out of @total.'));
    foreach (array_chunk($uids, 20) as $chunk) {
      $batch_builder->addOperation(
        [$this, 'batchSend'],
        [$chunk, $form_values['subject'], $form_values['body']]
      );
    }
    batch_set($batch_builder->toArray());
  }


  /**
   * Batch callback.
   */
  public function batchSend($uids, $subject, $body_field) {
    $users = User::loadMultiple($uids);
    foreach ($users as $account) {
      $this->mailPluginManager->mail(
        'cforge_group',
        'mass',
        $account->getEmail(),
        \Drupal::currentuser()->getPreferredLangcode(),
        [
          'subject' => $subject,
          'body' => check_markup($body_field['value'], $body_field['format']),
          'group' => $this->entity
        ]
      );
    }
  }

}
